/* $Id: angle.h 117 2008-08-06 14:53:40Z dirk $ */

/* GenLib2 - General Library 2
 * Copyright (C) 2005 Dirk Stallmann
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#ifndef GENLIB2_ANGLE_INCLUDED
#define GENLIB2_ANGLE_INCLUDED

#include <iomanip>
#include "uangle.h"
#include "dmsconv.h"

namespace GenLib2
{

/** Angle class.
 *  Principle: [measurement] = [value] + [unit]).
 *  @author Dirk Stallmann
 */
class Angle : public UnitAngle
{

public:
	/// default constructor
	Angle(bool dms=false) : UnitAngle(), _val(0), _is_dms(dms) {}
	
	/// constructor using symbol
	Angle(const string& s, bool dms=false) : UnitAngle(s), _val(0),
		_is_dms(dms) {}

	/// set decimal format
	void dec() { _is_dms = false; }

	/// set DMS format
	void dms() { _is_dms = true; }

	/// check DMS format
	bool is_dms() const { return _is_dms == true; }

	/// set value
	void set(double v) { _val = ubase(v); }

	/// get value
	double get() const { return conv(_val); }

	/// set value in radiant
	void rad(double v) { _val = v; }

	/// get value radiant
	double rad() const { return _val; }

	/// get value gon
	double deg() const { return _val * rho_deg ; }

	/// get value gon
	double gon() const { return _val * rho_gon ; }

	/// get unit
	const char* unit() const { return _symbol.c_str(); }
	
private:
	/// value in radiant
	double _val;

	/// DMS (true) or decimal (false)
	bool _is_dms;
};


// non-member implementations...

/// input operator
inline istream& operator>>(istream& is, Angle& a)
{
	double t = 0;

	if (a.is_dms())
		{
		double h = 0, m = 0, s = 0;
		is >> h >> m >> s;
		t = dms2dec(h, m, s);
		}
	else
		is >> t;

	a.set(t);

	return is;
}


/// output operator
inline ostream& operator<<(ostream& os, const Angle& a)
{
	if (a.is_dms())
		{
		double h = 0, m = 0, s = 0;
		dec2dms(a.get(), h, m, s);
		os.setf(ios::fixed, ios::floatfield);
		os << setw(4) << setprecision(0) << h << " ";
		os << setw(2) << m << " ";
		os << setw(8) << setprecision(5) << s;
		}
	else
		return os << a.get();

	return os;
}

} // namespace GenLib2

#endif /* GENLIB2_ANGLE_INCLUDED */

