/* $Id: sstransf.cpp 117 2008-08-06 14:53:40Z dirk $ */

/* GenLib2 - General Library 2
 * Copyright (C) 2005 Dirk Stallmann
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#include "form.h"
#include "sstransf.h"

namespace GenLib2
{

// constructor

SSTransform::SSTransform() : _dx(0.), _dy(0.), _dz(0.),
	_rot_matrix(), _scale(1.) { }


// set rotation matrix for coordinate transformation
// from a global into a local ellipsoidal system
/* hier oder besser RotMatrix-Klasse?
void SSTransform::set_rot_matrix(double phi, double lam)
{
	double sin_p = sin(phi);
	double cos_p = cos(phi);
	double sin_l = sin(lam);
	double cos_l = cos(lam);

	_rot_matrix(0,0) = -sin_l;
	_rot_matrix(0,1) = cos_l;
	_rot_matrix(0,2) = 0.;

	_rot_matrix(1,0) = -sin_p * cos_l;
	_rot_matrix(1,1) = -sin_p * sin_l;
	_rot_matrix(1,2) = cos_p;

	_rot_matrix(2,0) = cos_p * cos_l;
	_rot_matrix(2,1) = cos_p * sin_l;
	_rot_matrix(2,2) = sin_p;
}
*/

// set rotation matrix for rotation angles
// sequence: omega-phi-kappa

/*
void SSTransform::set_rot_matrix(double omega, double phi, double kappa)
{
	rot_matrix.setRot(omega, phi, kappa, opk);
	rot_matrix.trn();
}
*/


// set rotation matrix for small rotations
// The differential rotations have in the class RotMatrix3
// the opposite sign!

void SSTransform::set_rot_matrix(double eps_x, double eps_y, double eps_z)
{
	_rot_matrix(0,0) = 1.;
	_rot_matrix(0,1) =  eps_z;
	_rot_matrix(0,2) = -eps_y;

	_rot_matrix(1,0) = -eps_z;
	_rot_matrix(1,1) = 1.;
	_rot_matrix(1,2) =  eps_x;

	_rot_matrix(2,0) =  eps_y;
	_rot_matrix(2,1) = -eps_x;
	_rot_matrix(2,2) = 1.;
}


// read parameters

bool SSTransform::read(istream& is)
{
	//double omega = 0., phi = 0., kappa = 0.;
	double eps_x = 0., eps_y = 0., eps_z = 0.;

	is >> _dx >> _dy >> _dz;
	//is >> omega >> phi >> kappa;
	//_rot_matrix.setRot(omega, phi, kappa, opk);
	is >> eps_x >> eps_y >> eps_z;
	set_rot_matrix(eps_x, eps_y, eps_z);

	is >> _scale;

	return true;
}


// print parameters

void SSTransform::print(ostream& os) const
{
	Form fix3; fix3.fixed().width(8).precision(3);
	Form fix8; fix8.fixed().width(11).precision(8);

	os << "Translation in X :   " << fix3(_dx) << endl;
	os << "Translation in Y :   " << fix3(_dy) << endl;
	os << "Translation in Z :   " << fix3(_dz) << endl;
	os << "Rotation matrix  :\n";
	os << "  " << fix8(_rot_matrix(0,0))
		<< " " << fix8(_rot_matrix(0,1))
		<< " " << fix8(_rot_matrix(0,2)) << endl;
	os << "  " << fix8(_rot_matrix(1,0))
		<< " " << fix8(_rot_matrix(1,1))
		<< " " << fix8(_rot_matrix(1,2)) << endl;
	os << "  " << fix8(_rot_matrix(2,0))
		<< " " << fix8(_rot_matrix(2,1))
		<< " " << fix8(_rot_matrix(2,2)) << endl;
	os << "Scale            :   " << fix8(_scale) << endl;
}


// do transformation

void SSTransform::transform(const double x1, double y1, double z1,
double& x2, double& y2, double& z2) const
{
	x2 = _rot_matrix(0,0) * x1 + _rot_matrix(0,1) * y1
		+ _rot_matrix(0,2) * z1; 
	y2 = _rot_matrix(1,0) * x1 + _rot_matrix(1,1) * y1
		+ _rot_matrix(1,2) * z1; 
	z2 = _rot_matrix(2,0) * x1 + _rot_matrix(2,1) * y1
		+ _rot_matrix(2,2) * z1; 

	x2 *= _scale;
	y2 *= _scale;
	z2 *= _scale;

	x2 += _dx;
	y2 += _dy;
	z2 += _dz;
}


// do inverse transformation

void SSTransform::inv_transform(double x2, double y2, double z2,
double& x1, double& y1, double& z1) const
{
	double dx = x2 - _dx;
	double dy = y2 - _dy;
	double dz = z2 - _dz;

	x1 = _rot_matrix(0,0) * dx + _rot_matrix(1,0) * dy
		+ _rot_matrix(2,0) * dz; 
	y1 = _rot_matrix(0,1) * dx + _rot_matrix(1,1) * dy
		+ _rot_matrix(2,1) * dz; 
	z1 = _rot_matrix(0,2) * dx + _rot_matrix(1,2) * dy
		+ _rot_matrix(2,2) * dz; 

	x1 /= _scale;
	y1 /= _scale;
	z1 /= _scale;
}


// invert transformation parameters

void SSTransform::invert()
{
	// new translation
	double tmp_dx = _rot_matrix(0,0) * _dx + _rot_matrix(1,0) * _dy
		+ _rot_matrix(2,0) * _dz; 
	double tmp_dy = _rot_matrix(0,1) * _dx + _rot_matrix(1,1) * _dy
		+ _rot_matrix(2,1) * _dz; 
	double tmp_dz = _rot_matrix(0,2) * _dx + _rot_matrix(1,2) * _dy
		+ _rot_matrix(2,2) * _dz; 
	_dx = tmp_dx / -_scale;
	_dy = tmp_dy / -_scale;
	_dz = tmp_dz / -_scale;
	
	// new scale
	_scale = 1. / _scale;

	// new rotation matrix
	_rot_matrix.transpose();
}

} // namespace GenLib2
