/* $Id: vecmat.h 117 2008-08-06 14:53:40Z dirk $ */

/* GenLib2 - General Library 2
 * Copyright (C) 2005 Dirk Stallmann
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#ifndef GENLIB2_VECMAT_INCLUDED
#define GENLIB2_VECMAT_INCLUDED

#include <cassert>
#include <iostream>
#include "vector.h"
#include "matrix.h"

using namespace std;

namespace GenLib2
{

/// matrix-vector- multiplication function
template<class T>
void mult(Vector<T>& c, const Matrix<T>& A, const Vector<T>& b, bool clr=true)
{
	int m = A.rows();
	int n = A.cols();
	int o = b.size();

    // check dimensions
    assert(n == o);

	// set matrix dimensions (redimension only when required)
    c.newsize(m);

	// clear in advance
	if (clr) c.clear();

    for (register int i = 0; i < m; i++)
		{
		register T sum = (T)0;

		for	(register int j = 0; j < n; j++)
			{
			sum += A(i,j) * b(j);
			}

		c(i) += sum;
		}
}


/// matrix-vector multiplication function
template<class T>
Vector<T> mult(const Matrix<T>& A, const Vector<T>& b)
{
    Vector<T> c;
	mult(c, A, b, true);
	return c;
}


/// matrix-vector multiplication operator
template<class T>
Vector<T> operator*(const Matrix<T>& A, const Vector<T>& b)
{
    return mult(A,b);
}


/// vector-matrix multiplication function
template<class T>
void mult(Vector<T>& c, const Vector<T>& a, const Matrix<T>& B, bool clr=true)
{
	int n = a.size();
	int o = B.rows();
	int p = B.cols();

    // check dimensions
    assert(n == o);

	// set matrix dimensions (redimension only when required)
    c.newsize(p);

	// clear in advance
	if (clr) c.clear();

    for (register int i = 0; i < p; i++)
		{
		register T sum = (T)0;

		for	(register int j = 0; j < n; j++)
			{
			sum += a(j) * B(j,i);
			}

		c(i) += sum;
		}
}


/// vector-matrix multiplication function
template<class T>
Vector<T> mult(const Vector<T>& a, const Matrix<T>& B)
{
    Vector<T> c;
	mult(c, a, B, true);
	return c;
}


/// vector-matrix multiplication operator
template<class T>
Vector<T> operator*(const Vector<T>& a, const Matrix<T>& B)
{
    return mult(a,B);
}

} // namespace GenLib2

#endif
